/**
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 * 
 * Contact Information:
 * beck et al. projects GmbH
 * Theresienhoehe 13
 * 80336 Munich
 * Phone: +49-89-5442530
 * mailto: info@bea.de
 */
package de.bsvrz.ars.export.gui;

import java.awt.BorderLayout;
import java.awt.FlowLayout;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.ArrayList;
import java.util.List;

import javax.swing.BorderFactory;
import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTextField;
import javax.swing.ListSelectionModel;

import de.bsvrz.ars.export.Dataspecification;
import de.bsvrz.ars.export.exceptions.InvalidObjectException;
import de.bsvrz.dav.daf.main.DataDescription;
import de.bsvrz.dav.daf.main.config.Aspect;
import de.bsvrz.dav.daf.main.config.AttributeGroup;
import de.bsvrz.pat.sysbed.preselection.lists.PreselectionLists;
import de.bsvrz.pat.sysbed.preselection.lists.PreselectionListsListener;

/**
 * Dialog zur Datenauswahl. Setzt das Auswahl-Panel von K2S ein.
 * 
 * @author beck et al. projects GmbH
 * @author Phil Schrettenbrunner
 * @version $Revision: 1.1 $ / $Date: 2008/01/22 16:39:42 $ / ($Author: yvonnes $)
 * 
 */
public class DataSelectionDialog extends JDialog implements ActionListener, PreselectionListsListener {
    private static final long serialVersionUID = 8494762171841127798L;
    private JButton OkButton, CancelButton;
    private JPanel InputPanel;
    private JTextField Name;
    private PreselectionLists pl;
    private ExportFrame caller;
    private int elementId = -1;


    /**
     * Erstellt ein neues Datenauswahl-Fenster.
     * 
     * @param eFrame Das Aufrufende ExportFrame
     */
    public DataSelectionDialog(ExportFrame eFrame) {
        super((JFrame) eFrame, "Datenauswahl", true);
        caller = eFrame;
        initComponents();
        pack();
        setSize(600, 500);
    }


    /**
     * Behandelt die ActionEvents, wenn einer der beiden Buttons gedrückt wird.
     * 
     * @see java.awt.event.ActionListener#actionPerformed(java.awt.event.ActionEvent)
     */
    public void actionPerformed(ActionEvent ae) {
        if (ae.getActionCommand().equals(CancelButton.getActionCommand())) {
            dispose();
        } else if (ae.getActionCommand().equals(OkButton.getActionCommand())) {
            try {
                if (elementId < 0) {
                    // Hinzufügen
                    caller.addElement(new Dataspecification(pl.getSelectedObjects(), new DataDescription((AttributeGroup) pl.getSelectedAttributeGroups()
                            .get(0), (Aspect) pl.getSelectedAspects().get(0), (short) pl.getSimulationVariant()), Name.getText()));
                } else {
                    // Editieren
                    caller.editElement(elementId, new Dataspecification(pl.getSelectedObjects(), new DataDescription((AttributeGroup) pl
                            .getSelectedAttributeGroups().get(0), (Aspect) pl.getSelectedAspects().get(0), (short) pl.getSimulationVariant()), Name.getText()));
                }
            } catch (NullPointerException e) {
                // Ungültige Daten vom GTM Modul
                e.printStackTrace();
            } catch (InvalidObjectException e) {
                // Ungültige Daten vom GTM Modul
            }
            dispose();
        }
    }


    /**
     * Öffnet das Datenauswahlfenster. Da keine ID angegeben wurde, wird das alte Fenster
     * wiederverwendet (inkl. Auswahl)
     */
    public void open() {
        elementId = -1;
        setVisible(true);
    }


    /**
     * Öffnet das Datenauswahlfenster und wählt die Daten der durch die ID angegebenen
     * Datenspezifikation automatisch aus.
     * 
     * @param index Index.
     */
    @SuppressWarnings("unchecked")
    public void open(int index) {
        elementId = index;
        Dataspecification ds = caller.getDataspecification(index);
        List l = new ArrayList();
        l.add(ds.getAtg());
        pl.setPreselectedAttributeGroups(l);
        l = new ArrayList();
        l.add(ds.getAsp());
        pl.setPreselectedAspects(l);
        pl.setSimulationVariant(ds.getSimVar());
        pl.setPreselectedObjects((List) ds.getSystemObjects());
        Name.setText(ds.getName());
        setVisible(true);
    }


    /**
     * Initialisiert die Komponenten der GUI, zeigt jedoch noch nichts an.
     */
    private void initComponents() {
        getContentPane().setLayout(new BorderLayout());

        // ///// Dateneingabe
        InputPanel = new JPanel(new GridBagLayout());

        // Zeile Name
        Insets i = new Insets(0, 0, 0, 5);
        GridBagConstraints gbc = new GridBagConstraints();
        gbc.gridx = 0;
        gbc.gridy = 0;
        gbc.insets = i;
        gbc.fill = GridBagConstraints.HORIZONTAL;
        InputPanel.add(new JLabel("Name:", JLabel.RIGHT), gbc);

        gbc = (GridBagConstraints) gbc.clone();
        gbc.gridx++;
        gbc.weightx = 1;
        Name = new JTextField();
        InputPanel.add(Name, gbc);

        InputPanel.setBorder(BorderFactory.createEmptyBorder(7, 7, 7, 7));

        getContentPane().add(InputPanel, BorderLayout.NORTH);

        // Preselection
        pl = new PreselectionLists();
        pl.setAtgSelectionMode(ListSelectionModel.SINGLE_SELECTION);
        pl.setAspSelectionMode(ListSelectionModel.SINGLE_SELECTION);
        pl.setObjectTypeSelectionMode(ListSelectionModel.SINGLE_SELECTION);
        pl.setSimulationVariant(0);
        pl.showSimulationVariant();
        pl.setObjects(caller.getExportProfile().getAllObjects());
        pl.addPreselectionListener(this);
        getContentPane().add(pl, BorderLayout.CENTER);

        // ///// Button Panel
        JPanel ButtonPanel = new JPanel(new FlowLayout(FlowLayout.TRAILING));
        OkButton = new JButton("OK");
        OkButton.setEnabled(false);
        OkButton.addActionListener(this);
        ButtonPanel.add(OkButton);
        CancelButton = new JButton("Abbrechen");
        CancelButton.addActionListener(this);
        ButtonPanel.add(CancelButton);

        getContentPane().add(ButtonPanel, BorderLayout.SOUTH);
    }


    /** 
     * @see PreselectionListsListener#listSelectionChanged(PreselectionLists)
     */
    public void listSelectionChanged(PreselectionLists ps) {
        if (ps.getSelectedAspects().size() > 0 && ps.getSelectedAttributeGroups().size() > 0 && ps.getSelectedObjects().size() > 0)
            OkButton.setEnabled(true);
        else
            OkButton.setEnabled(false);
    }
}
