/**
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 * 
 * Contact Information:
 * beck et al. projects GmbH
 * Theresienhoehe 13
 * 80336 Munich
 * Phone: +49-89-5442530
 * mailto: info@bea.de
 */
package de.bsvrz.ars.export.gui;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.BorderFactory;
import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;

import de.bsvrz.ars.export.Positioner;
import de.bsvrz.ars.export.interfaces.ProgressListener;
import de.bsvrz.ars.export.prot.ProtocollerAdapter;

/**
 * Diese Klasse visualisiert den Exportforgang/Exportfortschritt
 * 
 * @author beck et al. projects GmbH
 * @author Phil Schrettenbrunner
 * @version $Revision: 1.1 $ / $Date: 2008/01/22 16:39:42 $ / ($Author: yvonnes $)
 */
public class ExportProgress extends JDialog implements ProgressListener, ActionListener {
    private static final long serialVersionUID = 5439567317477936223L;
    private long counter;
    private ProtocollerAdapter protocollerAdapter;
    private JButton cancelButton;
    private JLabel format;
    private JLabel dataSpec;
    private JLabel records;


    /**
     * Erzeugt ein neues graphisches Fortschrittsfenster mit Verweis auf den "Besitzer"
     * 
     * @param owner Besitzer des Dialoges
     */
    public ExportProgress(JFrame owner) {
        super(owner, "Exportvorgang läuft", true);
        initcomponents();
        setSize(400, 100);
        setPreferredSize(new Dimension(400, 135));
        pack();
        Positioner.center(owner, this);
    }


    /**
     * @see de.bsvrz.ars.export.interfaces.ProgressListener#init(de.bsvrz.ars.export.prot.ProtocollerAdapter)
     */
    public void init(ProtocollerAdapter pa) {
        counter = 0;
        protocollerAdapter = pa;
        setVisible(true);
    }


    /**
     * @see de.bsvrz.ars.export.interfaces.ProgressListener#setFormat(java.lang.String)
     */
    public void setFormat(String s) {
        format.setText(s);
    }


    /**
     * @see de.bsvrz.ars.export.interfaces.ProgressListener#recordReceived()
     */
    public void recordReceived() {
        counter++;
        updateCounter();
    }


    /**
     * @see de.bsvrz.ars.export.interfaces.ProgressListener#nextDataSpecification(int, int, boolean)
     */
    public void nextDataSpecification(int current, int max, boolean reset) {
        if (reset) {
            counter = 0;
            updateCounter();
        }
        updateDSpez(current, max);
    }


    /**
     * @see de.bsvrz.ars.export.interfaces.ProgressListener#done()
     */
    public void done() {
        dispose();
    }


    private void updateDSpez(int current, int max) {
        dataSpec.setText(current + " von " + max);
    }


    private void updateCounter() {
        records.setText("" + counter);
    }


    /**
     * Erstellt die Komponenten der GUI und ordnet sie an.
     */
    private void initcomponents() {
        getContentPane().setLayout(new BorderLayout());

        // /// Info Panel
        JPanel infoPanel = new JPanel(new GridLayout(3, 1));

        // Format Info
        FlowLayout fl = new FlowLayout(FlowLayout.LEFT);
        fl.setVgap(2);
        JPanel formatPanel = new JPanel(fl);
        formatPanel.add(new JLabel("Format: "));
        formatPanel.setBorder(BorderFactory.createEmptyBorder());
        format = new JLabel("");
        formatPanel.add(format);
        infoPanel.add(formatPanel);

        // DataSpecification Info
        JPanel dataSpecPanel = new JPanel(fl);
        dataSpecPanel.add(new JLabel("Datenspezifikation: "));
        dataSpecPanel.setBorder(BorderFactory.createEmptyBorder());
        dataSpec = new JLabel("...");
        dataSpecPanel.add(dataSpec);
        infoPanel.add(dataSpecPanel);

        // Record Info
        JPanel recordPanel = new JPanel(fl);
        recordPanel.add(new JLabel("Datensätze: "));
        recordPanel.setBorder(BorderFactory.createEmptyBorder());
        records = new JLabel("...");
        recordPanel.add(records);
        infoPanel.add(recordPanel);

        getContentPane().add(infoPanel, BorderLayout.NORTH);

        // /// Button Panel
        cancelButton = new JButton("Abbrechen");
        cancelButton.addActionListener(this);
        fl = new FlowLayout(FlowLayout.CENTER);
        fl.setVgap(8);
        JPanel buttonPanel = new JPanel(fl);
        buttonPanel.add(cancelButton);

        getContentPane().add(buttonPanel, BorderLayout.SOUTH);

    }


    /**
     * @see java.awt.event.ActionListener#actionPerformed(java.awt.event.ActionEvent)
     */
    public void actionPerformed(ActionEvent e) {
        if (protocollerAdapter != null) {
            cancelButton.setEnabled(false);
            protocollerAdapter.interrupt();
        }
    }
}
